

from apgl.util.Parameter import Parameter 
import numpy

class ErdosRenyiGenerator(object):
    '''
    A class which generates graphs according to the Erdos-Renyi Model. At construction time, it
    takes an empty graph, for which it creates edges.
    '''
    def __init__(self, graph):
        self.graph = graph 
        
    def generateGraph(self, p, edgeWeight=1, selfEdges=False):
        '''
        Create an Erdos-Renyi graph with edge probability p and given edgeWeight.
        For all pairs of vertices in the graph, an edge exists with probability p.

        :param p: the probability of an edge
        :type p: :class:`float`

        :param edgeWeight: the non-zero weight of newly created edges.
        :type edgeWeight: :class:`float`
        '''
        Parameter.checkFloat(p, 0.0, 1.0)

        numVertices = self.graph.getNumVertices()
        undirected = self.graph.isUndirected()

        P = numpy.random.rand(numVertices, numVertices)
        W = numpy.array(P < numpy.ones((numVertices, numVertices))*p, numpy.float64)

        if undirected == True:
            upTriInds = numpy.triu_indices(numVertices)
            diagW = numpy.diag(W)
            W[upTriInds] = 0
            W = W + W.T + numpy.diag(diagW)

        if not selfEdges:
            W[numpy.diag_indices(numVertices)] = 0 

        W = W*edgeWeight
        self.graph.setWeightMatrix(W)
            
        return self.graph
            
    def clusteringCoefficient(self, p):
        '''
        Returns the clustering coefficient for the graph created with edge probability
        p.

        :param p: The probability of an edge.
        :type p: :class:`float`
        '''
        Parameter.checkFloat(p, 0.0, 1.0)
        
        return p

    graph = None
    