

import numpy
import numpy.random as rand


class Latex(object):
    '''
    A class with some general useful functions for Latex output.
    '''
    def __init__(self):
        pass

    @staticmethod
    def array2DToLatex(X, precision=3):
        """
        Take a 2D array and print a Latex table
        """
        if X.ndim != 2:
            raise ValueError("Array must be one dimensional")

        n = X.shape[0]
        outputStr = ""

        for i in range(n):
            outputStr += Latex.array1DToLatex(X[i, :], precision) + "\\\\\n"

        return outputStr

    @staticmethod
    def array1DToLatex(X, precision=3):
        """
        Take a 1D numpy array and print in latex table format x1 & x2 .. xn
        """
        if X.ndim != 1:
            raise ValueError("Array must be one dimensional")

        n = X.shape[0]
        outputStr = ""

        if X.dtype == float:
            fmtStr = "%." + str(precision) + "f & "
            endFmtStr = "%." + str(precision) + "f"
        else:
            fmtStr = "%d & "
            endFmtStr = "%d"

        for i in range(0, n):
            if i != n-1:
                outputStr += fmtStr % X[i]
            else:
                outputStr += endFmtStr % X[i]

        return outputStr


    @staticmethod
    def meanStdArraysToLatex(means, stds):
        """
        A method which will print one line of a latex table using a 1D array of means
        and standard deviations. Prints in format mean1 (std1) & mean2 (std2) ...
        """
        if means.ndim != 1 or stds.ndim != 1:
            raise ValueError("Array must be one dimensional")
        if means.shape != stds.shape:
            raise ValueError("Arrays must be the same shape")

        n = means.shape[0]
        outputStr = ""

        for i in range(0, n):
            if i != n-1:
                outputStr += "%.3f(%.3f) & " % (means[i], stds[i])
            else:
                outputStr += "%.3f(%.3f)" % (means[i], stds[i])

        return outputStr

    @staticmethod
    def listToLatex(lst):
        """
        Just take a list and convert into a row of a latex table.
        """
        outputStr = ""

        for i in range(len(lst)):
            if i != len(lst)-1: 
                outputStr += str(lst[i]) + " & "
            else:
                outputStr += str(lst[i]) + "\\\\"

        return outputStr