'''
Created on 31 Jul 2009

@author: charanpal
'''

import numpy
import math 

class Parameter(object):
    def __init__(self):
        pass

    @staticmethod
    def checkClass(obj, objectType):
        """
        Check if an object belongs to a particular class
        """
        if not isinstance(obj, objectType):
            raise ValueError("Expecting object of type " + str(objectType) + " but received " + str(obj.__class__ ))


    @staticmethod
    def checkBoolean(val):
        if type(val) != bool:
            raise ValueError("Expecting boolean but received " + str(type(val)))

    @staticmethod
    def checkIndex(i, min, max):
        """
        This is a utility method to check that an index is in a range.
        """
        return Parameter.checkInt(i, min, max-1)

    @staticmethod
    def checkInt(i, min, max):
        """
        Test if i is between min inclusive and max exclusive, and throw an error if not
        """
        if not (type(min) in Parameter.intTypes or min == float("-inf")):
            raise ValueError("Minimum value is not an integer: " + str(min))
        if not (type(max) in Parameter.intTypes or max == float("inf")):
            raise ValueError("Maximum value is not an integer: " + str(max))
        if min > max:
            raise ValueError("Min is greater than max: [" + str(min) + ", " + str(max) + "]")

        #Weird bug with not testing correctly int32 types
        if math.trunc(i) != i:
            raise ValueError("Invalid parameter value (not int): " + str(i) + " is " + str(type(i)))
        if i < min or i > max:
            raise ValueError("Invalid parameter value: " + str(i) + " not in [" + str(min)+ ", " + str(max) + "]" )

    @staticmethod
    def checkFloat(i, min, max):
        "Test if i is between min and max inclusive, and throw an error if not"
        if type(min) not in Parameter.floatTypes:
            raise ValueError("Minimum value is not a float: " + str(min))
        if type(max) not in Parameter.floatTypes:
            raise ValueError("Maximum value is not a float: " + str(max))
        if min > max:
            raise ValueError("Min is greater than max: [" + str(min)+ ", " + str(max) + "]")

        if type(i) not in Parameter.floatTypes:
            raise ValueError("Invalid parameter value (not float): " + str(i) + " is " + str(type(i)))
        if i < min or i > max:
            raise ValueError("Invalid parameter value: " + str(i) + " not in [" + str(min)+ ", " + str(max) + "]" )

    @staticmethod
    def checkString(s, strList):
        if type(strList) != list:
            raise ValueError("Second parameters is required to be a list: " + str(strList))

        if type(s) != str or s not in strList:
            raise ValueError("Invalid string: " + str(s))

    @staticmethod
    def checkList(list, function, params):
        for i in list:
            function(i, *params)

    intTypes = [int, numpy.int8, numpy.int16, numpy.int32, numpy.int64]
    floatTypes = [float, numpy.float32, numpy.float64]