from apgl.graph import *
from apgl.viroscopy.model.HIVIndividual import HIVIndividual

"""
A HIVGraph is a SparseGraph except that its vertices are HIVIndividuals and
there are some useful method to find the S, I and R individuals. 
"""

class HIVGraph(SparseGraph):
    def __init__(self, numVertices, undirected=True, W=None):
        """
        Create a graph with the specified number of vertices, and choose whether
        edges are directed. 
        """
        vList = GeneralVertexList(numVertices)
        
        for i in range(numVertices):
            vertex = HIVIndividual()
            vList.setVertex(i, vertex)

        super(HIVGraph, self).__init__(vList, undirected, W)

    def getSusceptibleSet(self):
        susceptibleSet = set([])

        for i in range(self.getNumVertices()):
            if self.getVertex(i).isSusceptible():
                susceptibleSet.add(i)

        return susceptibleSet

    def getInfectedSet(self):
        infectedSet = set([])

        for i in range(self.getNumVertices()):
            if self.getVertex(i).isInfected():
                infectedSet.add(i)

        return infectedSet

    def getRemovedSet(self):
        removedSet = set([])

        for i in range(self.getNumVertices()):
            if self.getVertex(i).isRemoved():
                removedSet.add(i)

        return removedSet

        
        
