
from apgl.graph import *
from apgl.util import *
from apgl.viroscopy.model.HIVGraph import HIVGraph
from apgl.viroscopy.model.HIVEpidemicModel import HIVEpidemicModel
from apgl.viroscopy.model.HIVRates import HIVRates

import scipy 
import logging
import sys
import numpy
import pickle 

"""
This is the epidemic model for the HIV spread in cuba. 
"""

logging.basicConfig(stream=sys.stdout, level=logging.INFO)

#Total number of people in population
M = 10000
numInitialInfected = 5
defaultFreeDegree = 50

#The graph is one in which edges represent a contact
undirected = True
W = scipy.sparse.lil_matrix((M, M))
graph = HIVGraph(M, undirected, W=W)

for i in range(M):
    vertex = graph.getVertexList().getVertex(i)
    vertex.setFreeDegree(defaultFreeDegree)

    #Set the infection time of a number of individuals to 0 
    if i < numInitialInfected:
        vertex.setInfected(0.0)
    
#End time
T = 500
recordStep = 5 
rates = HIVRates(graph)
model = HIVEpidemicModel(graph, rates)

times, infectedIndices, removedIndices, graph = model.simulate(T, recordStep)

#Let's just save the graph here
#Could just save the detected individuals to mirror the real data 
outputDir = PathDefaults.getOutputDir() + "viroscopy/"

graphFileName = outputDir + "epidemicGraph.pkl"
graphFile = open(graphFileName, 'wb')
pickle.dump(graph, graphFile)
graphFile.close()

logging.info("Saved epidemic graph to file " + graphFileName)

indicesFileName = outputDir + "epidemicDetectIndices.pkl"
indicesFile = open(indicesFileName, 'wb')
pickle.dump(removedIndices, indicesFile)
indicesFile.close()

logging.info("Saved epidemic detected indices to file " + indicesFileName)

#TODO: 
#Inference - can compute which pairs of people are likely to have contact
#and also which ones get infected.