

import unittest
import logging
import sys 
from apgl.viroscopy.model.HIVEpidemicModel import HIVEpidemicModel
from apgl.viroscopy.model.HIVGraph import HIVGraph
from apgl.viroscopy.model.HIVRates import HIVRates
from apgl.graph import * 

class  HIVEpidemicModelTest(unittest.TestCase):
    def setUp(self):
        logging.basicConfig(stream=sys.stdout, level=logging.INFO)

        M = 100
        #Graph is one in which edges represent a contact
        undirected = True
        self.graph = HIVGraph(M, undirected)

        for i in range(M):
            vertex = self.graph.getVertex(i)
            vertex.setFreeDegree(5)

        rates = HIVRates(self.graph)
        self.model = HIVEpidemicModel(self.graph, rates)
        
    def testSimluate(self):
        T = 1

        self.graph.getVertex(0).setFreeDegree(5)
        self.graph.getVertex(0).setInfected(0.0)

        times, infectedIndices, removedIndices, graph = self.model.simulate(T)

        print(times)

        numInfects = 0
        for i in range(graph.getNumVertices()):
            if graph.getVertex(i).isInfected():
                numInfects += 1

        self.assertTrue(numInfects == 0 or times[len(times)-1] >= T)

        #TODO: Much better testing 

if __name__ == '__main__':
    unittest.main()

